# Generated by Django 2.2.9 on 2020-04-07 14:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('celeryanalytics', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='celerytaskcompleted',
            name='task',
            field=models.CharField(max_length=500),
        ),
        migrations.AlterField(
            model_name='celerytaskfailed',
            name='task',
            field=models.CharField(max_length=500),
        ),
        migrations.AddIndex(
            model_name='celerytaskcompleted',
            index=models.Index(fields=['task'], name='celeryanaly_task_b60e69_idx'),
        ),
        migrations.AddIndex(
            model_name='celerytaskcompleted',
            index=models.Index(fields=['time'], name='celeryanaly_time_77b42e_idx'),
        ),
        migrations.AddIndex(
            model_name='celerytaskfailed',
            index=models.Index(fields=['task'], name='celeryanaly_task_ae253d_idx'),
        ),
        migrations.AddIndex(
            model_name='celerytaskfailed',
            index=models.Index(fields=['time'], name='celeryanaly_time_bbde51_idx'),
        ),
    ]



# Generated by Django 3.1.1 on 2021-03-14 01:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('celeryanalytics', '0002_auto_20200407_1433'),
    ]

    operations = [
        migrations.AddField(
            model_name='celerytaskcompleted',
            name='runtime',
            field=models.FloatField(default=0),
        ),
        migrations.AddField(
            model_name='celerytaskfailed',
            name='runtime',
            field=models.FloatField(default=0),
        ),
    ]



from django.db import models


class CeleryTaskCompleted(models.Model):
    task = models.CharField(max_length=500)
    result = models.TextField()
    time = models.DateTimeField()
    runtime = models.FloatField(default=0)
    def __str__(self):
        return "%s completed at: %s" % (self.task, self.time.strftime("%Y-%m-%d %H:%M:%S"))

    class Meta:
        indexes = (
            models.Index(fields=['task']),
            models.Index(fields=['time'])
        )


class CeleryTaskFailed(models.Model):
    task = models.CharField(max_length=500)
    excep = models.TextField()
    trace = models.TextField()
    time = models.DateTimeField()
    runtime = models.FloatField(default=0)

    def __str__(self):
        return "%s failed at: %s with error %s" % (self.task, self.time.strftime("%Y-%m-%d %H:%M:%S"), self.excep)

    class Meta:
        indexes = (
            models.Index(fields=['task']),
            models.Index(fields=['time'])
        )




from django.core.management.base import BaseCommand
from celeryanalytics.tasks import run_housekeeping

class Command(BaseCommand):
    help = 'Run the Celery Analytics Housekeeping task.'

    def add_arguments(self, parser):
        parser.add_argument('--task', action='store_true',
                            help='Run update via Celery')

    def handle(self, *args, **options):
        if options.get("tasks", False):
            self.stdout.write("Cleaning up, Please wait this may take some time.")
            run_housekeeping()
        else:
            self.stdout.write("Sending Housekeeping task to the Celery Queue.")
            run_housekeeping.apply_async(priority=9)



from . import urls
from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

"""
class CeleryMenu(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(self, 'Celery',
                              'fas fa-address-book fa-fw',
                              'celery:celery_mon',
                              navactive=['celery:celery_mon'])

    def render(self, request):
        if request.user.is_superuser:
            return MenuItemHook.render(self, request)
        return ''




@hooks.register('menu_item_hook')
def register_menu():
    return CeleryMenu()


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'celery', r'^celery/')
"""


from celery.signals import task_failure, task_success, task_prerun
from .models import CeleryTaskCompleted, CeleryTaskFailed
from django.utils import timezone
from time import time
import datetime
import traceback as tb
import logging
from . import app_settings
logger = logging.getLogger(__name__)

task_starts = {}


def _calc_runtime(task_id):
    try:
        return time() - task_starts.pop(task_id)
    except KeyError:
        return -1


@task_failure.connect
def process_failure_signal(exception, traceback, sender, task_id,
                           signal, args, kwargs, einfo, **kw):
    logger.info("Celery task_failure! %s" % sender.__class__.__name__)
    runtime = _calc_runtime(task_id)
    CeleryTaskFailed.objects.create(task = sender.__class__.__name__,
                                    time=datetime.datetime.utcnow().replace(tzinfo=timezone.utc),
                                    runtime=runtime,
                                    excep=str(exception if exception else "Unknown"),
                                    trace=str(tb.format_exc() if traceback else "None"))


@task_success.connect
def celery_success_signal(sender, result=None, **kwargs):
    logger.info("Celery task_success! %s" % sender.__class__.__name__)
    runtime = _calc_runtime(sender.request.id)
    result = str(result)
    if app_settings.CA_RESULT_MAX_LEN > 0:
        result = result[:app_settings.CA_RESULT_MAX_LEN]
        
    CeleryTaskCompleted.objects.create(task=sender.__class__.__name__,
                                       result=str(result),
                                       runtime=runtime,
                                       time=datetime.datetime.utcnow().replace(tzinfo=timezone.utc))


@task_prerun.connect
def task_prerun_handler(signal, sender, task_id, task, args, kwargs, **extras):
    task_starts[task_id] = time()



from django.core.management.base import BaseCommand, CommandError
from django_celery_beat.models import CrontabSchedule, PeriodicTask

class Command(BaseCommand):
    help = 'Setup and Reset the Celery Analytics Tasks schedules'

    def add_arguments(self, parser):
        parser.add_argument('--inline', action='store_true',
                            help='Run update in this Console not via Celery')

    def handle(self, *args, **options):
        schedule_hk, _ = CrontabSchedule.objects.get_or_create(minute='0',
                                                                hour='11',
                                                                day_of_week='*',
                                                                day_of_month='*',
                                                                month_of_year='*',
                                                                timezone='UTC'
                                                                )


        housekeeping = PeriodicTask.objects.update_or_create(
            task='celeryanalytics.tasks.run_housekeeping',
            defaults={
                'crontab': schedule_hk,
                'name': 'Celery Analyitics Housekeeping',
                'enabled': True,
                'priority': 9
            }
        )

        self.stdout.write("Added housekeeping task to run daily at Down Time 1100 UTC")
        self.stdout.write("You can change the run time from within the periodic tasks menu in auth > admin > periodic tasks")


from celery import shared_task
from datetime import timedelta
from django.utils import timezone
from celeryanalytics.models import CeleryTaskCompleted, CeleryTaskFailed
from celeryanalytics.app_settings import CA_HOUSEKEEPING_DB_BACKLOG


@shared_task
def run_housekeeping():
    """
    Cleanup Database
    :return:
    """

    comp_qs = CeleryTaskCompleted.objects.filter(
        time__lte=timezone.now() - timedelta(days=CA_HOUSEKEEPING_DB_BACKLOG)
    )

    if comp_qs.exists():
        comp_qs._raw_delete(comp_qs.db)

    fail_qs = CeleryTaskFailed.objects.filter(
        time__lte=timezone.now() - timedelta(days=CA_HOUSEKEEPING_DB_BACKLOG)
    )

    if fail_qs.exists():
        fail_qs._raw_delete(fail_qs.db)



from django.contrib import admin

# Register your models here.
from .models import CeleryTaskCompleted, CeleryTaskFailed


class FailedAdmin(admin.ModelAdmin):
    list_display=('task', 'time', 'excep')
    date_hierarchy = 'time'
    list_filter=('task',)

class CompleteAdmin(admin.ModelAdmin):
    list_display=('task', 'time')
    date_hierarchy = 'time'
    list_filter=('task',)

admin.site.register(CeleryTaskCompleted, CompleteAdmin)
admin.site.register(CeleryTaskFailed, FailedAdmin)



# Generated by Django 2.0.5 on 2019-02-02 09:06

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CeleryTaskCompleted',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('task', models.TextField()),
                ('result', models.TextField()),
                ('time', models.DateTimeField()),
            ],
        ),
        migrations.CreateModel(
            name='CeleryTaskFailed',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('task', models.TextField()),
                ('excep', models.TextField()),
                ('trace', models.TextField()),
                ('time', models.DateTimeField()),
            ],
        ),
    ]
